export declare type Operation<Input = void, Output = void> = (data: Input, 
/**
 * The abort signal enter an aborted state if a new operation starts before
 * the previous one finishes, or if the component is unmounted before the
 * operation finishes.
 */
abortSignal: AbortSignal) => Promise<Output>;
export declare type OperationAct<Input = void, Output = void> = (data: Input) => Promise<Output>;
export declare type UseOperationResult<Input = void, Output = void> = readonly [
    trigger: (data: Input) => Promise<Output>,
    pending: boolean,
    output?: Output
];
/**
 * The operation function itself is passed in as an argument, so that we're able
 * to type the output parameter.
 *
 * Note that if you'd like to execute an arbitrary function in response to an
 * event, you can make an operation whose input is itself an async function.
 */
export declare function useOperation<Input = void, Output = void>(operation: Operation<Input, Output>, deps?: any[]): UseOperationResult<Input, Output>;
